function [synch,asynch] = corr2dhet(data1,data2,ref1,ref2)
%CORR2DHET Two-dimensional (2D) hetero-spectral correlation analysis of spectra.
%
%   [SYNCH, ASYNCH] = CORR2D(DATA1, DATA2), returns a synchronous (SYNCH) and
%   asynchronous (ASYNCH) spectra, whereas the dynamic spectra were calculated
%   by means of an average spectrum. DATA is a column matrix of spectra, i.e.
%   the first column is a scale and the following columns are intensities. The
%   number of spectra must be the same in DATA1 and DATA2 but the number of
%   datapoints does not have to be the same.
%
%   [SYNCH, ASYNCH] = CORR2D(DATA1, DATA2, REF1, REF2), returns a synchronous
%   and asynchronous spectra. REF is a reference spectrum (double column
%   matrix) which is subtracted for creation of the dynamic spectra. If REF = 0
%   (the scalar value) then the original spectra are identical with the
%   dynamics spectra. If REF = 1 the first or if REF = 2 the last spectrum of
%   DATA is taken as the reference spectrum. REF = 3 denotes standard
%   calculation of the dynamic spectra by means of an average spectrum.
%
%   The function was performed on the base of articles of Prof. Noda [1,2,3].
%   Nice simple explanation of the 2D analysis can be found in [3]. Summary of
%   examples and practical usage of 2D analysis in vibrational spectroscopy can
%   be found in [4].
%
%   See also:
%   CORR2D, CORR2DGUI.
%
%   References:
%   [1] I. Noda, Appl. Spectrosc. 47 (1993) 1329.
%   [2] I. Noda, Appl. Spectrosc. 54 (2000) 994.
%   [3] I. Noda, General theory of two-dimensional (2D) analysis, in Handbook
%       of Vibrational Spectroscopy (J.M. Chalemers, P.R. Griffiths, Eds.), John
%       Wiley, New York 2002, p. 2123.
%   [4] Y. Ozaki, 2D correlation spectroscopy in vibrational spectroscopy, in
%       Handbook of Vibrational Spectroscopy (J.M. Chalemers, P.R. Griffiths,
%       Eds.), John Wiley, New York 2002, p. 2135.
%   
%   Copyright (c) 2003 by Vladimir Kopecky Jr.
%   Revision: 1.01  Date: 21. 9. 2003

%   Subfunction & Callbacks:
%   reference - calculates reference spectrum for dynamic spectra

if nargin < 2 | nargin > 4 
    error('The procedure requires two, three or four input arguments.');
    return;
end
[m,n] = size(data1);
[o,p] = size(data2);
if n < 3 | m == 1 | p < 3 | o == 1
    errordlg('The set of spectra is strange and improper for analysis','Computational Error');
    return;
end
if n ~= p
    errordlg('The number of analyzed spectra in spectral sets must be the same.','Computational Error');
    return;
end
%calculation of dynamic spectra (i.e. normalization)
if nargin == 2 %the reference spectrum is an average spectrum
    ref1 = mean(data1(:,2:n)');
    ref1 = ref1';
    ref2 = mean(data2(:,2:n)');
    ref2 = ref2';    
elseif nargin == 3
    ref1 = reference(data1,ref1);
    ref2 = mean(data2(:,2:n)');
    ref2 = ref2';
else
    ref1 = reference(data1,ref1);
    ref2 = reference(data2,ref2);
end
for i=2:n
   data1(:,i) = data1(:,i) - ref1;
   data2(:,i) = data2(:,i) - ref2;
end
%direct calculation of synchronous spectrum
synch = data1(:,2:n) * data2(:,2:n)'/(n-2); 
%discrete Hilbert-Noda transformation matrix
N = zeros(n-1);
for i=2:(n-1)
    N(1,i) = 1/(pi * (i-1));
end
for i=2:(n-2)
    for j=i:(n-1)
        N(i,j) = N(i-1,j-1);
    end;
end
for i=2:(n-1)
    for j=1:i
        N(i,j) = -N(j,i);
    end;
end
%asychronous spectrum by Hilbert-Noda transform
asynch = data1(:,2:n) * N * data2(:,2:n)'/(n-2);

%______________________________________________________________________________

function ref = reference(data,ref)
%REFERENCE Subfunction for calculating reference spectrum for normalization.

[m,n] = size(data);
[o,p] = size(ref);
if o == 1 & p == 1
    if ref == 0 %without a reference spectrum
        ref = zeros(m,1);
    elseif ref == 1 %the first spectrum of the data set is the reference
        ref = data(:,2);
    elseif ref == 2 %the last spectrum of the data set is the reference
        ref = data(:,n);
    elseif ref == 3 %the reference spectrum is an average spectrum
        ref = mean(data(:,2:n)');
        ref = ref2';
    else
        error('The reference input argument is strage.');
        return;
    end
elseif p > 2 | (p == 1 & o > 1)
    errordlg('The reference spectrum is not a single spectrum made of two columns.','Computational Error');
    return;
else
    if m == o
        if sum(data(:,1) == ref(:,1)) == m
            ref(:,1) = [];
        else
            errordlg('The input refernce spectrum is shifted.','Computational Error');
            return;                
        end
    else
        errordlg('The input refernce spectrum has a different length.','Computational Error');
        return;
    end
end